@@ -137,7 +137,7 @@ def build_filter(model, key, value, joins=None):
                     elif exop == 'gte':
                         exp = expression.extract(op.upper(), field) >= int(value)
                     elif exop == 'in':
-                        exp = expression.extract(op.upper(), field).in_(map(lambda x:int(x),
+                        exp = expression.extract(op.upper(), field).in_(map(lambda x: int(x),
                                                                             value if isinstance(value, (list, tuple))
                                                                             else str2list(value)))
                     elif exop == 'range':
@@ -168,6 +168,7 @@ def build_filter(model, key, value, joins=None):
 
 def build_order_by(cls, order_by):
     """build_order_by: build order by criterias with the given list order_by in strings."""
+
     def _gen_order_by(c, by):
         is_desc = False
         if by and by.startswith('-'):
@@ -194,6 +195,7 @@ def build_order_by(cls, order_by):
 
 def find_join_loads(cls, extend_fields):
     """find_join_loads: find the relationships from extend_fields which we can call joinloads for EagerLoad..."""
+
     def _relations_(c, exts):
         if not exts:
             return None
@@ -243,7 +245,6 @@ def query_reparse(query, internal_filters=None):
 
 
 def restruct_ext_fields(cls, extend_fields):
-
     def _f_(s):
         ss = s.split('.', 1)
         logger.debug('_f_: %s', ss)
@@ -316,19 +317,24 @@ def exception_wapper(f):
         try:
             result = f(self, *args, **kwargs)
         except sa_exc.IntegrityError, e:
+            logger.exception(e)
             self._dbsession_.rollback()
-            raise InvalidData(detail=u'%s' % e)
+            raise InvalidData(detail=str(e))
         except sa_exc.SQLAlchemyError, e:
+            logger.exception(e)
             self._dbsession_.rollback()
-            raise FatalError(detail=u'%s' % e)
+            raise FatalError(detail=str(e))
         except ExpressError, e:
+            logger.exception(e)
             self._dbsession_.rollback()
             raise e
         except Exception, e:
+            logger.exception(e)
             self._dbsession_.rollback()
-            raise FatalError(detail=u'%s' % e)
+            raise FatalError(detail=str(e))
         else:
             return result
+
     return wrapper_f
 
 
@@ -397,6 +403,45 @@ class ExpressController(RestController):
         logger.debug('[%s]_after>>>>>: args=%s, %s', self.__class__.__name__, args, kw)
         #super(ExpressController, self)._after(*args, **kw)
 
+    ####################################################################################################################
+    def _before_read(self, pk=None, query=None, **kwargs):
+        logger.debug('[%s]_before_read> pk=%s, query=%s',
+                     self.__class__.__name__, pk, query)
+
+    def _before_update(self, inst, arguments, **kwargs):
+        logger.debug('[%s]_before_update> inst=%s, arguments=%s',
+                     self.__class__.__name__,
+                     inst, arguments)
+
+    def _before_create(self, objects, **kwargs):
+        logger.debug('[%s]_before_create> objects=%s',
+                     self.__class__.__name__,
+                     objects)
+
+    def _before_delete(self, inst, **kwargs):
+        logger.debug('[%s]_before_delete> inst=%s',
+                     self.__class__.__name__,
+                     inst)
+
+    def _after_read(self, inst, **kwargs):
+        logger.debug('[%s]_after_read> inst=%s',
+                     self.__class__.__name__, inst)
+
+    def _after_update(self, inst, **kwargs):
+        logger.debug('[%s]_after_update> inst=%s',
+                     self.__class__.__name__, inst)
+
+    def _after_create(self, objects, **kwargs):
+        logger.debug('[%s]_after_create> objects=%s',
+                     self.__class__.__name__,
+                     objects)
+
+    def _after_delete(self, deletes, **kwargs):
+        logger.debug('[%s]_after_delete> deletes=%s',
+                     self.__class__.__name__, deletes)
+
+    ####################################################################################################################
+
     def _check_permission(self, seckey):
         permissions = getattr(self, '_permissions_', None)
         if permissions is None:
@@ -407,7 +452,7 @@ class ExpressController(RestController):
         try:
             predicate.check_authorization(tg.request.environ)
         except NotAuthorizedError as e:
-            reason = unicode(e)
+            reason = str(e)
             if hasattr(self, '_failed_authorization'):
                 # Should shortcircuit the rest, but if not we will still
                 # deny authorization
@@ -503,12 +548,12 @@ class ExpressController(RestController):
                 if orderbys:
                     inst = inst.order_by(*orderbys)
             if limit >= 0:
-                inst = inst.slice(begin, begin+limit)  # inst[begin:begin+limit]
+                inst = inst.slice(begin, begin + limit)  # inst[begin:begin+limit]
             result['__count'] = inst.count()
             result[self._model_.__name__] = serialize(self._model_,
                                                       inst,
                                                       include_fields=include_fields, extend_fields=extend_fields)
-             # list(inst.values(*[getattr(self._model_, x) for x in include_fields]))
+            # list(inst.values(*[getattr(self._model_, x) for x in include_fields]))
         else:
             logger.debug("Inst >>> %s", inst)
             logger.debug("Include Fields: %s", include_fields)
@@ -518,7 +563,7 @@ class ExpressController(RestController):
                 result['__count'] = len(objs)
             else:
                 result[self._model_.__name__] = objs
-            # dict([(k, getattr(inst, k)) for k in include_fields])
+                # dict([(k, getattr(inst, k)) for k in include_fields])
         return result
 
     def _validate_object_data(self, object_data):
@@ -652,6 +697,7 @@ class ExpressController(RestController):
         join_loads = find_join_loads(self._model_, extend_fields)
         join_loads = [joinedload(x) for x in join_loads] if join_loads else None
         logger.debug('join_loads: %s', join_loads)
+        self._before_read(pk=pk, query=query)
         if pk is not None:
             inst = self._dbsession_.query(self._model_).options(*join_loads).get(pk) if join_loads \
                 else self._dbsession_.query(self._model_).get(pk)
@@ -662,6 +708,7 @@ class ExpressController(RestController):
             if join_loads:
                 inst = inst.options(*join_loads)
         logger.debug('Inst: %s', type(inst))
+        self._after_read(inst)
         return self._serialize(inst, include_fields=include_fields,
                                exclude_fields=exclude_fields,
                                extend_fields=extend_fields,
@@ -705,7 +752,7 @@ class ExpressController(RestController):
                     logger.debug('pks = %s', pks)
                     new_obj_datas = filter(lambda m: isinstance(m, dict) and related_class_pk_name not in m, v)
                     if pks:
-                        exits_objs = self._dbsession_.query(related_class).\
+                        exits_objs = self._dbsession_.query(related_class). \
                             filter(getattr(related_class, related_class_pk_name).in_(pks)).all()
                         logger.debug('exits_objs = %s', exits_objs)
                 elif isinstance(v, dict):
@@ -729,9 +776,10 @@ class ExpressController(RestController):
                 if exits_objs:
                     related_objs = exits_objs
                 if new_objs:
-                    related_objs = new_objs if not related_objs else related_objs+new_objs
+                    related_objs = new_objs if not related_objs else related_objs + new_objs
                 logger.debug('[%s]related_objs: %s', k, related_objs)
                 setattr(obj, k, related_objs)
+            self._before_create(obj, **data)
             return obj
 
         if isinstance(arguments, (list, tuple)):
@@ -739,9 +787,11 @@ class ExpressController(RestController):
             self._dbsession_.add_all(objects)
         else:
             objects = _do_create_obj(arguments)
+            self._before_create(objects, **arguments)
             self._dbsession_.add(objects)
         logger.debug('objects: %s', objects)
         self._dbsession_.flush()
+        self._after_create(objects)
         return objects, list(set(ext_flds))
 
     @exception_wapper
@@ -759,6 +809,7 @@ class ExpressController(RestController):
             if not isinstance(arguments, dict) or not arguments:
                 raise InvalidData()
             arguments = self._validate_object_data(self._encode_object_data(arguments))
+            self._before_update(inst, arguments)
             for k, v in arguments.items():
                 if self._readonly_fields_ and k in self._readonly_fields_:
                     raise InvalidData(detail='Column(%s) is read-only!' % k)
@@ -770,12 +821,14 @@ class ExpressController(RestController):
             if not isinstance(arguments, dict) or not arguments:
                 raise InvalidData()
             arguments = self._validate_object_data(self._encode_object_data(arguments))
+            self._before_update(inst, arguments)
             for k, v in arguments.items():
                 if self._readonly_fields_ and k in self._readonly_fields_:
                     raise InvalidData(detail='Column(%s) is read-only!' % k)
             inst.update(arguments)
             result = inst
         self._dbsession_.flush()
+        self._after_update(inst)
         return result, ext_flds
 
     @exception_wapper
@@ -788,19 +841,22 @@ class ExpressController(RestController):
             inst = self._dbsession_.query(self._model_).get(pk)
             if not inst:
                 raise NotFound()
+            self._before_delete(inst)
             result = {'__count': 1,
                       'deleted': [{self._model_.__table__.primary_key.columns.keys()[0]: pk}],
                       '__model': self._model_.__name__}
             self._dbsession_.delete(inst)
         else:
             inst = self._query(query)
+            self._before_delete(inst)
             result = map(lambda x: x._asdict(), inst.values(*self._model_.__table__.primary_key.columns.values()))
-                #map(lambda x: {self._model_.__table__.primary_key.columns.keys()[0]: x},
-                #         inst.values(self._model_.__table__.primary_key.columns.values()[0]))
+            #map(lambda x: {self._model_.__table__.primary_key.columns.keys()[0]: x},
+            #         inst.values(self._model_.__table__.primary_key.columns.values()[0]))
             result = {'__count': len(result),
                       'deleted': result,
                       '__model': self._model_.__name__}
-            inst.delete()
+            inst.delete(synchronize_session='fetch')
+        self._after_delete(result['deleted'])
         return result
 
     ###################################################################################################################
@@ -826,7 +882,7 @@ class ExpressController(RestController):
         except Exception, e:
             logger.exception(u'>>> %s', e)
             #abort(404, u"Object not found!", comment=u"%s" % e)
-            raise FatalError(detail=u'%s' % e, title=u'Unknown error!')
+            raise FatalError(detail=str(e), title=u'Unknown error!')
         else:
             return result
 
@@ -845,7 +901,7 @@ class ExpressController(RestController):
             raise ne
         except Exception, e:
             logger.exception(u'>>> %s', e)
-            raise FatalError(detail=u'%s' % e)
+            raise FatalError(detail=str(e))
         else:
             return result
 
@@ -873,7 +929,7 @@ class ExpressController(RestController):
     #         raise ne
     #     except Exception, e:
     #         logger.exception(u'>>> %s', e)
-    #         raise ExpressError(detail=u'%s' % e)
+    #         raise ExpressError(detail=str(e))
     #     return self._serialize(result, extend_fields=ext_fields)
     #
     # @expose('json')
@@ -892,7 +948,7 @@ class ExpressController(RestController):
     #         raise ne
     #     except Exception, e:
     #         logger.exception(u'>>> %s', e)
-    #         raise ExpressError(detail=u'%s' % e)
+    #         raise ExpressError(detail=str(e))
     #     else:
     #         return self._serialize(result, extend_fields=ext_fields)
 
@@ -915,7 +971,7 @@ class ExpressController(RestController):
                 raise ne
             except Exception, e:
                 logger.exception(u'>>> %s', e)
-                raise FatalError(detail=u'%s' % e)
+                raise FatalError(detail=str(e))
         else:
             try:
                 self._check_permission('create')
@@ -925,7 +981,7 @@ class ExpressController(RestController):
                 raise ne
             except Exception, e:
                 logger.exception(u'>>> %s', e)
-                raise FatalError(detail=u'%s' % e)
+                raise FatalError(detail=str(e))
         return self._serialize(result, extend_fields=ext_fields)
 
     @expose('json')
@@ -947,7 +1003,7 @@ class ExpressController(RestController):
             raise ne
         except Exception, e:
             logger.exception(u'>>> %s', e)
-            raise FatalError(detail=u'%s' % e)
+            raise FatalError(detail=str(e))
         else:
             return self._serialize(result, extend_fields=ext_fields)
 
@@ -992,6 +1048,6 @@ class ExpressController(RestController):
             raise ne
         except Exception, e:
             logger.exception(u'>>> %s', e)
-            raise FatalError(detail=u'%s' % e)
+            raise FatalError(detail=str(e))
         else:
             return result
\ No newline at end of file
